class Node:
    def __init__(self, data=None, left=None, right=None):
        self.__data = data
        self.__left = left
        self.__right = right

    @property
    def data(self):
        return self.__data

    @data.setter
    def data(self, data):
        self.__data = data

    @property
    def left(self):
        return self.__left

    @left.setter
    def left(self, ptr):
        self.__left = ptr

    @property
    def right(self):
        return self.__right

    @right.setter
    def right(self, ptr):
        self.__right = ptr

    @classmethod
    def chain(cls, cur, node):
        try:
            assert node.data != cur.data
        except Exception:
            raise Exception("this number is already exist")
        if node.data < cur.data:
            if cur.left is None:
                cur.left = node
            else:
                cls.chain(cur.left, node)
        else:
            if cur.right is None:
                cur.right = node
            else:
                cls.chain(cur.right, node)



from .utils.node import Node


class SList:
    def __init__(self):
        self.__head = None
    
    def __str__(self):
        sl = "List : ["
        tmp = self.__head
        while tmp:
            sl += str(tmp.data)
            if tmp.next:
                sl += ", "
            tmp = tmp.next
        return sl + ']'

    def __len__(self):
        size = 0
        tmp = self.__head
        while tmp:
            tmp = tmp.next
            size += 1
        return size

    @property
    def first(self):
        try:
            return self.__head.data
        except Exception as ex:
            print("Error, %s" %ex, end=', ')

    @property
    def last(self):
        try:
            tmp = self.__head
            while tmp.next:
                tmp = tmp.next
            return tmp.data
        except Exception as ex:
            print("Error, %s" %ex, end=', ')
    
    def empty(self):
        return self.__head is None

    def clear(self):
        while not self.empty():
            ptr = self.__head
            self.__head = self.__head.next
            del ptr

    def append(self, data):
        node = Node(data)
        if self.empty():
            self.__head = node
        else:
            if data < self.__head.data:
                node.next = self.__head
                self.__head = node
            elif data == self.__head.data:
                node.next = self.__head.next
                self.__head.next = node
            else:
                tmp = self.__head
                cur = None
                while tmp.next and data >= tmp.data:
                    cur = tmp
                    tmp = tmp.next
                if data < tmp.data:
                    cur.next = node
                    node.next = tmp
                else:
                    node.next = tmp.next
                    tmp.next = node
                del tmp, cur

    def remove(self, pos=0):
        try:
            tmp = self.__head
            count = 0
            if pos == 0:
                ptr = self.__head
                data = self.__head.data
                self.__head = self.__head.next
                del ptr
                return data
            elif pos == -1:
                while tmp.next:
                    cur = tmp
                    tmp = tmp.next
                data = tmp.data
                cur.next = None
                del tmp
                return data
            while count < pos:
                cur = tmp
                tmp = tmp.next
                count += 1
            data = tmp.data
            cur.next = tmp.next
            del tmp
            return data
        except Exception as ex:
            print("Error, %s" %ex)

    def delete(self, data):
        try:
            while self.__head.data == data:
                ptr = self.__head
                self.__head = self.__head.next
                del ptr
            cur = self.__head
            tmp = cur.next
            while tmp:
                if tmp.data == data:
                    ptr = tmp
                    tmp = tmp.next
                    cur.next = tmp
                    del ptr
                else:
                    cur = tmp
                    tmp = tmp.next
        except Exception as ex:
            print("Error, %s" %ex)

    def find(self, data):
        count = 0
        tmp = self.__head
        while tmp:
            if tmp.data == data:
                count += 1
            tmp = tmp.next
        return count

    @classmethod
    def merge(cls, sl1, sl2):
        try:
            lst = SList()
            tmp1 = sl1.__head
            tmp2 = sl2.__head
            while tmp1:
                lst.append(tmp1.data)
                tmp1 = tmp1.next
            while tmp2:
                lst.append(tmp2.data)
                tmp2 = tmp2.next
            del tmp1, tmp2
            return lst
        except Exception as ex:
            print("Error, %s" %ex)

    @classmethod
    def swap(cls, sl1, sl2):
        bid = SList()
        while not sl1.empty():
            bid.append(sl1.remove())
        while not sl2.empty():
            sl1.append(sl2.remove())
        while not bid.empty():
            sl2.append(bid.remove())
        del bid



from .utils import change
from .utils.dnode import Node


class DList:
    def __init__(self):
        self.__head = None
        self.__tail = None

    def __str__(self):
        l = "DList : ["
        tmp = self.__head
        while tmp:
            l += str(tmp.data)
            if tmp.next:
                l += " <=> "
            tmp = tmp.next
        return l + "]"

    def __len__(self):
        size = 0
        tmp = self.__head
        while tmp:
            tmp = tmp.next
            size += 1
        return size

    @property
    def first(self):
        try:
            return self.__head.data
        except Exception as ex:
            print("Error, %s" %ex, end=', ')

    @property
    def last(self):
        try:
            return self.__tail.data
        except Exception as ex:
            print("Error, %s" %ex, end=', ')

    def empty(self):
        return self.__head is None

    def clear(self):
        while not self.empty():
            ptr = self.__head
            self.__head = self.__head.next
            del ptr
    
    def add_begin(self, data):
        node = Node(data)
        if self.empty():
            self.__head = self.__tail = node
        else:
            node.next=  self.__head
            self.__head.prev = node
            self.__head = node

    def add_fin(self, data):
        node = Node(data)
        if self.empty():
            self.__head=  self.__tail = node
        else:
            node.prev = self.__tail
            self.__tail.next = node
            self.__tail = node

    def insert(self, data, pos=0):
        node = Node(data)
        if self.empty():
            self.__head = self.__tail = node
        elif pos == 0:
            node.next = self.__head
            self.__head.prev = node
            self.__head = node
        elif pos == -1:
            self.add_fin(data)
        else:
            tmp = self.__head
            count = 0
            while tmp and count < pos:
                cur = tmp
                tmp = tmp.next
                count += 1
            cur.next = node
            node.prev = cur
            node.next = tmp

    def remove(self, pos=0):
        try:
            tmp = self.__head
            count = 0
            if pos == 0:
                ptr = self.__head
                data = self.__head.data
                self.__head = self.__head.next
                self.__head.prev= None
                del ptr
                return data
            elif pos == -1:
                while tmp.next:
                    cur = tmp
                    tmp = tmp.next
                data = tmp.data
                cur.next = None
                self.__tail = cur
                del tmp
                return data
            while count < pos:
                cur = tmp
                tmp = tmp.next
                count += 1
            nxt = tmp.next
            data = tmp.data
            cur.next = tmp.next
            nxt.prev = cur
            del tmp
            return data
        except Exception as ex:
            print("Error, %s" %ex)
    
    def delete(self, data):
        try:
            while self.__head.data == data:
                ptr = self.__head
                self.__head = self.__head.next
                self.__head.prev = None
                del ptr
            cur = self.__head
            tmp = cur.next
            while tmp:
                if tmp.data == data:
                    ptr = tmp
                    tmp = tmp.next
                    cur.next = tmp
                    del ptr
                else:
                    cur = tmp
                    tmp = tmp.next
            self.__tail = cur
        except Exception as ex:
            print("Error, %s" %ex)
    
    def find(self, data):
        count = 0
        tmp = self.__head
        while tmp:
            if tmp.data == data:
                count += 1
            tmp = tmp.next
        return count

    def reverse(self):
        prv = self.__head
        tmp = prv.next
        prv.next = None
        prv.prev = tmp
        while tmp:
            nxt = tmp.next
            tmp.next = prv
            tmp.prev = nxt
            prv = tmp
            tmp = nxt
        self.__head = prv

    def sort(self):
        tmp1 = self.__head
        tmp2 = tmp1.next
        while tmp1.next:
            while tmp2:
                if tmp1.data > tmp2.data:
                    change(tmp1, tmp2)
                tmp2 = tmp2.next
            tmp1 = tmp1.next
            tmp2 = tmp1.next
        del tmp1, tmp2

    def exchange(self, n):
        for i in range(n):
            self.add_fin(self.remove())

    @classmethod
    def merge(cls, dl1, dl2):
        try:
            lst = DList()
            tmp1 = dl1.__head
            tmp2 = dl2.__head
            while tmp1:
                lst.add_fin(tmp1.data)
                tmp1 = tmp1.next
            while tmp2:
                lst.add_fin(tmp2.data)
                tmp2 = tmp2.next
            del tmp1, tmp2
            return lst
        except Exception as ex:
            print("Error, %s" %ex)

    @classmethod
    def swap(cls, dl1, dl2):
        bid = DList()
        while not dl1.empty():
            bid.add_fin(dl1.remove())
        while not dl2.empty():
            dl1.add_fin(dl2.remove())
        while not bid.empty():
            dl2.add_fin(bid.remove())
        del bid



class Node:
    def __init__(self, data=None, next=None, prev=None):
        self.__data = data
        self.__next = next
        self.__prev = prev

    @property
    def data(self):
        return self.__data

    @data.setter
    def data(self, data):
        self.__data = data

    @property
    def next(self):
        return self.__next

    @next.setter
    def next(self, ptr):
        self.__next = ptr

    @property
    def prev(self):
        return self.__prev

    @prev.setter
    def prev(self, ptr):
        self.__prev = ptr



from .stack import Stack
from .utils import change
from .utils.node import Node


class PQueue:
    def __init__(self):
        self.__front = None
        self.__back = None

    def __str__(self):
        q = "Queue : ["
        tmp = self.__front
        while tmp:
            q += str(tmp.data)
            if tmp.next:
                q += " | "
            tmp = tmp.next
        return q + "]"

    def __len__(self):
        size = 0
        tmp = self.__front
        while tmp:
            size += 1
            tmp = tmp.next
        return size

    @property
    def front(self):
        try:
            return self.__front.data
        except Exception as ex:
            print("Error, %s" %ex, end=', ')

    @property
    def back(self):
        try:
            return self.__back.data
        except Exception as ex:
            print("Error, %s" %ex, end=', ')

    def empty(self):
        return self.__front is None

    def clear(self):
        while not self.empty():
            ptr = self.__front
            self.__front = self.__front.next
            del ptr

    def enqueue(self, data):
        node = Node(data)
        if self.empty():
            self.__front = self.__back = node
        else:
            if self.__front.data > data:
                node.next = self.__front
                self.__front = node
            elif self.__back.data < data:
                self.__back.next = node
                self.__back = node
            else:
                tmp = self.__front
                while tmp.next and tmp.data < data:
                    cur = tmp
                    tmp = tmp.next
                node.next = tmp
                cur.next = node
                if node.next is None:
                    self.__back = node

    def dequeue(self):
        ptr = self.__front
        data = ptr.data
        self.__front = self.__front.next
        del ptr
        return data

    def find(self, data):
        count = 0
        tmp = self.__front
        while tmp:
            if tmp.data == data:
                count += 1
            tmp = tmp.next
        return count

    def reverse(self):
        stck = Stack()
        while not self.empty():
            stck.push(self.enqueue())
        while not stck.empty():
            self.enqueue(stck.pop())
        return self

    def sort(self):
        tmp1 = self.__front
        tmp2 = tmp1.next
        while tmp1.next:
            while tmp2:
                if tmp1.data > tmp2.data:
                    change(tmp1, tmp2)
                tmp2 = tmp2.next
            tmp1 = tmp1.next
            tmp2 = tmp1.next
        del tmp1, tmp2

    def exchange(self, n):
        for i in range(n):
            self.enqueue(self.dequeue())

    @classmethod
    def merge(cls, q1, q2):
        try:
            que = PQueue()
            tmp1 = q1.__front
            tmp2 = q2.__front
            while tmp1:
                que.enqueue(tmp1.data)
                tmp1 = tmp1.next
            while tmp2:
                que.enqueue(tmp2.data)
                tmp2 = tmp2.next
            del tmp1, tmp2
            return que
        except Exception as ex:
            print("Error, %s" % ex)

    @classmethod
    def swap(cls, q1, q2):
        bid = PQueue()
        while not q1.empty():
            bid.enqueue(q1.dequeue())
        while not q2.empty():
            q1.enqueue(q2.dequeue())
        while not bid.empty():
            q2.enqueue(bid.dequeue())
        del bid



from .utils import change
from .utils.node import Node


class Stack:
    def __init__(self):
        self.__top = None

    def __str__(self):
        s = "Stack : ["
        tmp = self.__top
        while tmp:
            s += str(tmp.data)
            if tmp.next:
                s += "\n         "
            tmp = tmp.next
        return s + "]"

    def __len__(self):
        size = 0
        tmp = self.__top
        while tmp:
            size += 1
            tmp = tmp.next
        return size

    @property
    def top(self):
        try:
            return self.__top.data
        except Exception as ex:
            print("Error, %s" %ex, end=', ')

    def empty(self):
        return self.__top is None

    def clear(self):
        while not self.empty():
            ptr = self.__top
            self.__top = self.__top.next
            del ptr

    def push(self, data):
        node = Node(data, self.__top)
        self.__top = node

    def pop(self):
        ptr = self.__top
        data = ptr.data
        self.__top = self.__top.next
        del ptr
        return data

    def find(self, data):
        count = 0
        tmp = self.__top
        while tmp:
            if tmp.data == data:
                count += 1
            tmp = tmp.next
        return count

    def reverse(self):
        stck = Stack()
        while not self.empty():
            stck.push(self.pop())
        return stck

    def sort(self):
        tmp1 = self.__top
        tmp2 = tmp1.next
        while tmp1.next:
            while tmp2:
                if tmp1.data > tmp2.data:
                    change(tmp1, tmp2)
                tmp2 = tmp2.next
            tmp1 = tmp1.next
            tmp2 = tmp1.next
        del tmp1, tmp2

    def exchange(self, n):
        bid1 = Stack()
        bid2 = Stack()
        for i in range(n):
            bid1.push(self.pop())
        while not self.empty():
            bid2.push(self.pop())
        while not bid1.empty():
            self.push(bid1.pop())
        while not bid2.empty():
            self.push(bid2.pop())

    @classmethod
    def merge(cls, s1, s2):
        try:
            stck = Stack()
            tmp1 = s1.__top
            tmp2 = s2.__top
            while tmp1:
                stck.push(tmp1.data)
                tmp1 = tmp1.next
            while tmp2:
                stck.push(tmp2.data)
                tmp2 = tmp2.next
            del tmp1, tmp2
            return stck
        except Exception as ex:
            print("Error, %s" %ex)

    @classmethod
    def swap(cls, s1, s2):
        bid1 = Stack()
        bid2 = Stack()
        while not s1.empty():
            bid1.push(s1.pop())
        while not s2.empty():
            bid2.push(s2.pop())
        while not bid1.empty():
            s2.push(bid1.pop())
        while not bid2.empty():
            s1.push(bid2.pop())
        del bid1, bid2



from .node import Node
from .dnode import Node
from .tnode import Node
from .pair import Pair


def change(node1, node2):
    bid = node1.data
    node1.data = node2.data
    node2.data = bid
    del bid



class Pair:
    def __init__(self, first=None, second=None):
        self.__first = first
        self.__second = second

    def __str__(self):
        p = "Pair : ["
        if self.__first and self.__second:
            p += str(self.__first)
            p += ", "
            p += str(self.__second)
        return p + "]"

    @property
    def first(self):
        try:
            return self.__first
        except Exception as ex:
            print("Error, %s" % ex)

    @first.setter
    def first(self, data):
        self.__first = data

    @property
    def second(self):
        try:
            return self.__second
        except Exception as ex:
            print("Error, %s" % ex)

    @second.setter
    def second(self, data):
        self.__second = data

    @classmethod
    def make_pair(cls, first=None, second=None):
        pair = Pair()
        pair.__first = first
        pair.__second = second
        return pair



from pyfds.utils.tnode import Node


class BST:
    def __init__(self):
        self.__root = None
        self.__number_of_nodes = 0

    def __gototarget(self, tmp, data):
        while tmp:
            if data == tmp.data:
                break
            elif data < tmp.data:
                tmp = tmp.left
            else:
                tmp = tmp.right
        return tmp

    def __change_parents_child(self, data, p, node):
        if data < p.data:
            p.left = node
        else:
            p.right = node

    def __delete(self, data, tmp):
        p = self.parent(data)
        if p:
            self.__change_parents_child(data, p, tmp.left)
        else:
            self.__root = None
        del p

    def __min(self, data):
        tmp = self.__root
        tmp = self.__gototarget(tmp, data)
        while tmp.left:
            tmp = tmp.left
        return tmp

    def __max(self, data):
        tmp = self.__root
        tmp = self.__gototarget(tmp, data)
        while tmp.right:
            tmp = tmp.right
        return tmp

    def empty(self):
        return self.__root is None

    def clear(self):
        def _clear(root):
            if root:
                _clear(root.right)
                _clear(root.left)
                root.right = None
                root.left = None
                del root
        _clear(self.__root)
        self.__root = None

    def pre_order(self):
        def _pre_order(tmp):
            if tmp:
                print(tmp.data, end=', ')
                _pre_order(tmp.left)
                _pre_order(tmp.right)
        _pre_order(self.__root)
        if self.__root:
            print("end")

    def in_order(self):
        def _pre_order(tmp):
            if tmp:
                _pre_order(tmp.left)
                print(tmp.data, end=', ')
                _pre_order(tmp.right)
        _pre_order(self.__root)
        if self.__root:
            print("end")

    def post_order(self):
        def _pre_order(tmp):
            if tmp:
                _pre_order(tmp.left)
                _pre_order(tmp.right)
                print(tmp.data, end=', ')
        _pre_order(self.__root)
        if self.__root:
            print("end")

    @property
    def number_of_nodes(self):
        return self.__number_of_nodes

    @number_of_nodes.setter
    def number_of_nodes(self, num):
        self.__number_of_nodes += num

    @property
    def max(self):
        root = self.__root
        while root.right:
            root = root.right
        return root.data

    @property
    def min(self):
        root = self.__root
        while root.left:
            root = root.left
        return root.data

    def append(self, data):
        node = Node(data)
        if self.empty():
            self.__root = node
        else:
            Node.chain(self.__root, node)
        self.__number_of_nodes += 1

    def find(self, data):
        def _search(root):
            while root:
                if data == root.data:
                    return root
                elif data < root.data:
                    root = root.left
                else:
                    root = root.right
            return None
        return _search(self.__root)

    def parent(self, data):
        s = self.find(data)
        if s:
            if data == self.__root.data:
                return None
            else:
                tmp = self.__root
                p = None
                while tmp.data != data:
                    p = tmp
                    if data < tmp.data:
                        tmp = tmp.left
                    elif data > tmp.data:
                        tmp = tmp.right
                return p
        return None

    def successor(self, data):
        tmp = self.__root
        tmp = self.__gototarget(tmp, data)
        if tmp:
            if tmp.right:
                return self.__min(tmp.right.data)
            else:
                p = self.parent(data)
                while p.data < tmp.data:
                    if tmp == self.__root:
                        return None
                    else:
                        p = self.parent(p.data)
                return p
        return None

    def predecessor(self, data):
        tmp = self.__root
        tmp = self.__gototarget(tmp, data)
        if tmp:
            if tmp.left:
                return self.__max(tmp.left.data)
            else:
                p = self.parent(data)
                while p.data > tmp.data:
                    if tmp == self.__root:
                        return None
                    else:
                        p = self.parent(p.data)
                return p
        return None

    def delete(self, data):
        tmp = self.__root
        tmp = self.__gototarget(tmp, data)
        if tmp:
            if not tmp.left and not tmp.right:
                self.__delete(data, tmp)
            elif tmp.left and not tmp.right:
                self.__delete(data, tmp)
            elif not tmp.left and tmp.right:
                self.__delete(data, tmp)
            else:
                s = self.successor(data)
                p = self.parent(s.data)
                p.left = None
                s.left = tmp.left
                s.right = p
                p = self.parent(data)
                if p:
                    self.__change_parents_child(data, p, s)
                else:
                    self.__root = s
                del s, p
            d = tmp.data
            tmp.left = tmp.right = None
            del tmp
            self.__number_of_nodes -= 1
            return d
        return None
